/***************************************************************************
**
**  This file is part of millenium3.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2015-11-17
**  Copyright: 2015
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#include "MilleniumBuffer.h"

/*!
  \class MilleniumBuffer MilleniumBuffer.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
MilleniumBuffer::MilleniumBuffer(int fileDescriptor)
  : DynamicBuffer(fileDescriptor)
{
  TRACE;
  _outputFile.setBaseName("solaire");
  if(!_outputFile.open()) {
    Log::write(0, "Cannot open data output file\n");
  }
  statusHeader();
}

int MilleniumBuffer::bytesAvailable(char * buffer, int bytesCount)
{
  TRACE;
  int bytesRead=0;
  while(bytesRead<bytesCount) {
    // Scan for a line
    int newBytesRead;
    for(newBytesRead=bytesRead; newBytesRead<bytesCount && buffer[newBytesRead]!='\n'; newBytesRead++) {}
    if(newBytesRead==bytesCount) {
      return bytesRead; // partial line
    }
    // Full line available
    //printf("Full line '%.*s'\n", newBytesRead-1, buffer+bytesRead);
    if(!parse(buffer+bytesRead, newBytesRead-1)) {
      Log::write(0, "Error parsing response\n");
    }
    bytesRead=newBytesRead+1;
  }
  return bytesRead;
}

bool MilleniumBuffer::parse(char * buffer, int count)
{
  TRACE;
  if(strncmp(buffer, ":040310", 7)!=0) {
    Log::write(1, "Missing ':040310'\n");
    return false;
  }
  Log::write(2, "Received %i characters\n", count);
  //Log::write(2, "%.*s\n", buffer, count);
  buffer+=7;
  bool ok=true;
  for(int i=0; i<32; i+=4) {
    char  * c=_data.raw+(i >> 1);
    c[1]=ascii2bin(buffer[i], ok) << 4;
    c[1]+=ascii2bin(buffer[i+1], ok);
    c[0]=ascii2bin(buffer[i+2], ok) << 4;
    c[0]+=ascii2bin(buffer[i+3], ok);
    if(!ok) {
      Log::write(1, "Error for index %i\n", i+7);
      return false;
    }
    Log::write(2, "reading value %hhx\n", c);
  }
  saveStatus();
  Leds::flash(2, 100, true);
  return true;
}

char MilleniumBuffer::ascii2bin(char c, bool &ok)
{
  switch(c) {
  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
    return c-'0';
  case 'A':
  case 'B':
  case 'C':
  case 'D':
  case 'E':
  case 'F':
    return c-('A'-10);
  default:
    Log::write(1, "Unexpected character '%c'(%hhx)\n", c, c);
    ok=false;
    return 0;
  }
}

void MilleniumBuffer::statusHeader()
{
  TRACE;
  _outputFile.write("# time tHaut tMilieu tBas tSol tBois tSejour "
                    " vitSol cSol vSol cBois  cPlancher cSdb  cSanitaire vSanitaire  res alarme "
                    " mInTh mInTb mOutTh mOutTb "
                    " muChauffage muHorsGel muCanicule\n");
}

void MilleniumBuffer::saveStatus()
{
  TRACE;
  // Get shift between local time and UTC
  tzset();
  _outputFile.write("%i %.1f %.1f %.1f %.1f %.1f %.1f  %i %i %i %i  %i %i  %i %i  %i %i  %i %i %i %i  %i %i %i\n",
             time(0)-timezone+daylight*3600,
             0.1*_data.values.temperatureTamponHaut,
             0.1*_data.values.temperatureTamponMilieu,
             0.1*_data.values.temperatureTamponBas,
             0.1*_data.values.temperatureSolaire,
             0.1*_data.values.temperaturePoele,
             0.1*_data.values.temperatureSejour,
             _data.values.vitesseSolaire,
             _data.values.c_solaire ? 1 : 0,
             _data.values.v_tampon ? 1 : 0,
             _data.values.c_poele ? 1 : 0,
             _data.values.c_plancher ? 1 : 0,
             _data.values.c_sdb ? 1 : 0,
             _data.values.c_sanitaire ? 1 : 0,
             _data.values.v_sanitaire ? 1 : 0,
             _data.values.res ? 1 : 0,
             _data.values.alarme ? 1 : 0,
             _data.values.m_in_th ? 1 : 0,
             _data.values.m_in_tb ? 1 : 0,
             _data.values.m_out_th ? 1 : 0,
             _data.values.m_out_tb ? 1 : 0,
             _data.values.mu_chauffage ? 1 : 0,
             _data.values.mu_hors_gel ? 1 : 0,
             _data.values.mu_canicule ? 1 : 0);
}
